#include "stdafx.h"

#include "ExplosionParameters.h"
#include "ExplosionParametersReader.h"

ExplosionParameters::ExplosionParameters()
	:
	simulationWidth(256),
	simulationHeight(512),
	simulationDepth(1),

	fuelAreaCenterX(0.0f),
	fuelAreaCenterY(0.8f),
	fuelAreaCenterZ(0.5),

	sparkPositionX(0.0f),
	sparkPositionY(0.8f),
	sparkPositionZ(0.5),

	fuelAreaRadiusInTexels(24),
	framesToDrawFuel(20),

	buoyancy(3.0f),
	gravity(0.3f),

	burningRate(0.1f),
	stoichiometricMixture(800.0f),
	outputHeat(4.50f),

	fuelDissipation(0.05f),
	sootDissipation(0.3f),
	tempDissipation(5.6f),

	fuelDiffusion(3.2f),
	sootDiffusion(15.7f),
	tempDiffusion(17.5f),

	thermalExpansionRate(0.3f),
	vorticityConfinementStrength(9.0f),

	timeStep(1.0f/32.0f),

	sparkTemp(1.01f),
	lowerIgnitionLimit(1.0f),
	initialFuelDensity(1.0f),	

	densityDiffusionJacobiIterations(20),
	pressureGradientJacobiIterations(40),

	simulate3D(false),
	simulateFreely(true),
	renderDebuggedParticles(false),


	smokeParticleMassLightPercentage(20),
	smokeParticleMassLightBeginValue(0.4f),
	smokeParticleMassLightEndValue(1.5f),
	smokeParticleMassHeavyBeginValue(2.0f),
	smokeParticleMassHeavyEndValue(6.0f),

	fireParticleMassLightPercentage(80),
	fireParticleMassLightBeginValue(0.1f),
	fireParticleMassLightEndValue(2.0f),
	fireParticleMassHeavyBeginValue(2.0f),
	fireParticleMassHeavyEndValue(4.0f),

	vortexParticleCount(0),
	vortexParticleMaxStrength(0),
	vortexParticleMaxRadius(0),

	initialParticleAreaRadius(0.4f)
{	
}


void ExplosionParameters::InitFromFile(std::string filename)
{
	ExplosionParametersReader reader(filename, this);
}


ExplosionParameters::~ExplosionParameters()
{
}

//int ExplosionParameters::GetVortexParticleCount() {
//	return vortexParticleCount;
//}
//
//float ExplosionParameters::GetVortexParticleMaxStrength() {
//	return vortexParticleMaxStrength;
//}
//
//float ExplosionParameters::GetVortexParticleMaxRadius() {
//	return vortexParticleMaxRadius;
//}

int ExplosionParameters::GetDensityDiffusionJacobiIterations() const
{
	return densityDiffusionJacobiIterations;
}

int ExplosionParameters::GetPressureGradientJacobiIterations() const
{
	return pressureGradientJacobiIterations;
}

int ExplosionParameters::GetSimulationWidth() const
{
	return simulationWidth;
}

int ExplosionParameters::GetSimulationHeight() const
{
	return simulationHeight;
}

int ExplosionParameters::GetSimulationDepth() const
{
	return simulationDepth;
}

float ExplosionParameters::GetFuelAreaCenterX() const
{
	return fuelAreaCenterX;
}

float ExplosionParameters::GetFuelAreaCenterY() const
{
	return fuelAreaCenterY;
}
int ExplosionParameters::GetFuelAreaRadiusInTexels() const
{
	return fuelAreaRadiusInTexels;
}

int ExplosionParameters::GetFramesToDrawFuel() const
{
	return framesToDrawFuel;
}

float ExplosionParameters::GetSparkPositionX() const
{
	return sparkPositionX;
}

float ExplosionParameters::GetSparkPositionY() const
{
	return sparkPositionY;
}

float ExplosionParameters::GetSparkPositionZ() const
{
	return sparkPositionZ;
}

float* ExplosionParameters::GetTexelSizes() const 
{
	static float texelSizes[4];
	texelSizes[0] = 1.0f / GetSimulationWidth();
	texelSizes[1] = 1.0f / GetSimulationHeight();
	texelSizes[2] = 1.0f / GetSimulationDepth();
	texelSizes[3] = 0;
	return texelSizes;
}

float* ExplosionParameters::GetSimulationDims() const
{
	static float gridSizes[4];
	gridSizes[0] = (float)GetSimulationWidth();
	gridSizes[1] = (float)GetSimulationHeight();
	gridSizes[2] = (float)GetSimulationDepth();
	gridSizes[3] = 0;
	return gridSizes;
}

float ExplosionParameters::GetBuoyancy() const
{
	return buoyancy;
}

float ExplosionParameters::GetGravity() const
{
	return gravity;
}

float ExplosionParameters::GetInitialFuelDensity() const
{
	return initialFuelDensity;
}

float ExplosionParameters::GetLowerIgnitionLimit() const
{
	return lowerIgnitionLimit;
}

float ExplosionParameters::GetBurningRate() const
{
	return burningRate;
}

float ExplosionParameters::GetStoichiometricMixture() const
{
	return stoichiometricMixture;
}

float ExplosionParameters::GetOutputHeat() const
{
	return outputHeat;
}

float ExplosionParameters::GetFuelDissipation() const
{
	return fuelDissipation;
}

float ExplosionParameters::GetSootDissipation() const
{
	return sootDissipation;
}

float ExplosionParameters::GetTempDissipation() const
{
	return tempDissipation;
}

float ExplosionParameters::GetFuelDiffusion() const
{
	return fuelDiffusion;
}

float ExplosionParameters::GetSootDiffusion() const
{
	return sootDiffusion;
}

float ExplosionParameters::GetTempDiffusion() const
{
	return tempDiffusion;
}

float ExplosionParameters::GetThermalExpansionRate() const
{
	return thermalExpansionRate;
}

float ExplosionParameters::GetVorticityConfinementStrength() const
{
	return vorticityConfinementStrength;
}

float ExplosionParameters::GetTimeStep() const
{
	return timeStep;
}

ExplosionParameters& ExplosionParameters::Instance()
{
	static ExplosionParameters params;
	return params;
}

float ExplosionParameters::GetSparkTemp() const
{
	return sparkTemp;
}

float* ExplosionParameters::GetDissipation() const
{
	static float dissipation[4];
	dissipation[0] = GetFuelDissipation();
	dissipation[1] = GetSootDissipation();
	dissipation[2] = GetTempDissipation();
	dissipation[3] = 0;
	return dissipation;
}
