#pragma once
#include <string>

class ExplosionParameters
{
public:
	static ExplosionParameters& Instance();

	void InitFromFile(std::string filename);


	int GetSimulationWidth() const;
	int GetSimulationHeight() const;
	int GetSimulationDepth() const; // Only used for 3D simulations

	virtual float* GetTexelSizes() const;
	float* GetSimulationDims() const;

	float GetBuoyancy() const;
	float GetGravity() const;

	float GetInitialFuelDensity() const;
	int GetFramesToDrawFuel() const;
	float GetSparkTemp() const;

	float GetLowerIgnitionLimit() const;
	float GetBurningRate() const;
	float GetStoichiometricMixture() const;
	float GetOutputHeat() const;

	float* GetDissipation() const;
	float GetFuelDissipation() const;
	float GetSootDissipation() const;
	float GetTempDissipation() const;

	float GetFuelDiffusion() const;
	float GetSootDiffusion() const;
	float GetTempDiffusion() const;

	float GetThermalExpansionRate() const;

	float GetVorticityConfinementStrength() const;

	float GetTimeStep() const;

	float GetFuelAreaCenterX() const;
	float GetFuelAreaCenterY() const;
	float GetFuelAreaCenterZ() const; // Only used for 3D simulations
	int GetFuelAreaRadiusInTexels() const;

	float GetSparkPositionX() const;
	float GetSparkPositionY() const;
	float GetSparkPositionZ() const; // Only used for 3D simulations

	int GetDensityDiffusionJacobiIterations() const;
	int GetPressureGradientJacobiIterations() const;


	// Setters
	void SetSimulationWidth(int w) { simulationWidth = w; }
	void SetSimulationHeight(int h) { simulationHeight = h; }
	void SetSimulationDepth(int depth) { simulationDepth= depth; }

	void SetBuoyancy(float f) { buoyancy = f; }
	void SetGravity(float f) { gravity = f; }

	void SetInitialFuelDensity(float f) { initialFuelDensity = f; }
	void SetFramesToDrawFuel(int i) { framesToDrawFuel = i; }
	void SetSparkTemp(float f) { sparkTemp = f; }

	void SetLowerIgnitionLimit(float f) { lowerIgnitionLimit = f; }
	void SetBurningRate(float f) { burningRate = f; }
	void SetStoichiometricMixture(float f) { stoichiometricMixture = f; }
	void SetOutputHeat(float f) { outputHeat = f; }

	void SetFuelDissipation(float f) { fuelDissipation = f; }
	void SetSootDissipation(float f) { sootDissipation = f; }
	void SetTempDissipation(float f) { tempDissipation = f; }

	void SetFuelDiffusion(float f) { fuelDiffusion = f; }
	void SetSootDiffusion(float f) { sootDiffusion = f; }
	void SetTempDiffusion(float f) { tempDiffusion = f; }

	void SetThermalExpansionRate(float f) { thermalExpansionRate = f; }

	void SetVorticityConfinementStrength(float f) { vorticityConfinementStrength = f; }

	void SetTimeStep(float f) { timeStep = f; }

	void SetFuelAreaCenterX(float f) { fuelAreaCenterX = f; }
	void SetFuelAreaCenterY(float f) { fuelAreaCenterY = f; }
	void SetFuelAreaCenterZ(float z) { fuelAreaCenterZ = z; }

	void SetFuelAreaRadiusInTexels(int i) { fuelAreaRadiusInTexels = i; }

	void SetSparkPositionX(float f) { sparkPositionX = f; }
	void SetSparkPositionY(float f) { sparkPositionY = f; }
	void SetSparkPositionZ(float z) { sparkPositionZ = z; }

	void SetDensityDiffusionJacobiIterations(int i) { densityDiffusionJacobiIterations = i; }
	void SetPressureGradientJacobiIterations(int i) { pressureGradientJacobiIterations = i; }

	void SetSimulateFreely(bool b) { simulateFreely = b; }
	bool GetSimulateFreely() const { return simulateFreely; }

	bool ShouldSimulate3D() const { return simulate3D; }
	void SetSimulate3D(bool b) { simulate3D = b; }

	bool ShouldRenderDebuggedParticles() const { return renderDebuggedParticles; }
	void SetRenderDebuggedParticles(bool b) { renderDebuggedParticles = b; }

protected:
	ExplosionParameters();
	~ExplosionParameters();
	
	int simulationWidth;
	int simulationHeight;
	int simulationDepth;


	float fuelAreaCenterX;
	float fuelAreaCenterY;
	float fuelAreaCenterZ;

	int fuelAreaRadiusInTexels;
	int framesToDrawFuel;

	float sparkPositionX;
	float sparkPositionY;
	float sparkPositionZ;

	float buoyancy;
	float gravity;

	float initialFuelDensity;
	float sparkTemp;

	float lowerIgnitionLimit;
	float burningRate;
	float stoichiometricMixture;
	float outputHeat;

	float fuelDissipation;
	float sootDissipation;
	float tempDissipation;

	float fuelDiffusion;
	float sootDiffusion;
	float tempDiffusion;

	float thermalExpansionRate;

	float vorticityConfinementStrength;

	float timeStep;
	int densityDiffusionJacobiIterations;
	int pressureGradientJacobiIterations;

	bool simulateFreely;
	bool simulate3D;
	bool renderDebuggedParticles;


public: // There's no "generate getter and setter" here :(

	float smokeParticleMassLightPercentage;
	float smokeParticleMassLightBeginValue;
	float smokeParticleMassLightEndValue;
	float smokeParticleMassHeavyBeginValue;
	float smokeParticleMassHeavyEndValue;

	float fireParticleMassLightPercentage;
	float fireParticleMassLightBeginValue;
	float fireParticleMassLightEndValue;
	float fireParticleMassHeavyBeginValue;
	float fireParticleMassHeavyEndValue;

	int vortexParticleCount;
	float vortexParticleMaxStrength;
	float vortexParticleMaxRadius;

	float initialParticleAreaRadius;
};
