#include "stdafx.h"

#include "ExplosionParametersReader.h"

//#include <algorithm>
#include <cassert>
#include <fstream>
#include <iostream>
//#include <limits>
#include <sstream>
#include <vector>
#include "DXUT.h" // for the debug output thingy

ExplosionParametersReader::ExplosionParametersReader(std::string filename, ExplosionParameters* explosionParameters)
	:
	explosionParameters(explosionParameters)
{
	std::ifstream file;
	file.open(filename.c_str());

	if (file) {
		parseStream(file);
	} else {
		assert(!"Could not find configuration file");
	}

	file.close();
}

ExplosionParametersReader::~ExplosionParametersReader(void)
{
}

void ExplosionParametersReader::parseStream(std::istream &is)
{
	std::string line;

	while (std::getline(is, line))
		parseLine(line);
}

std::vector<std::string> tokenize(const std::string &line)
{
	std::vector<std::string> tokens;

	std::string::size_type pos = 0;

	while (pos != std::string::npos) {
		pos = line.find_first_not_of(" ", pos);

		if (pos != std::string::npos) {
			std::string::size_type current = pos;

			pos = line.find_first_of(" ", pos);

			tokens.push_back(line.substr(current, pos == std::string::npos ? pos : pos - current));
		}
	}

	return tokens;
}

template <typename Target>
Target lexical_cast(const std::string &source)
{
	std::stringstream ss;
	ss << source;
	Target t;
	ss >> t;
	return t;
}

std::string concat(std::vector<std::string> &parameters) 
{
	std::string parameterString;
	std::vector<std::string>::const_iterator iter = parameters.begin();

	while (iter != parameters.end()) {
		parameterString += *iter;

		if (++iter != parameters.end())
			parameterString += ' ';
	}
	return parameterString;
}

void ExplosionParametersReader::parseLine(const std::string &line)
{
	assert(line.find('\n') == std::string::npos);

	std::vector<std::string> parameters = tokenize(line);

	if (!parameters.empty()) {
		std::string param = parameters[0];
		parameters.erase(parameters.begin());
		
		std::string arguments = concat(parameters);
		if (param == "simulationWidth")
			explosionParameters->SetSimulationWidth(lexical_cast<int>(arguments));
		else if (param == "simulationHeight")
			explosionParameters->SetSimulationHeight(lexical_cast<int>(arguments));
		else if (param == "simulationDepth")
			explosionParameters->SetSimulationDepth(lexical_cast<int>(arguments));

		else if (param == "fuelAreaCenterX")
			explosionParameters->SetFuelAreaCenterX(lexical_cast<float>(arguments));
		else if (param == "fuelAreaCenterY")
			explosionParameters->SetFuelAreaCenterY(lexical_cast<float>(arguments));
		else if (param == "fuelAreaCenterZ")
			explosionParameters->SetFuelAreaCenterZ(lexical_cast<float>(arguments));

		else if (param == "sparkPositionX")
			explosionParameters->SetSparkPositionX(lexical_cast<float>(arguments));
		else if (param == "sparkPositionY")
			explosionParameters->SetSparkPositionY(lexical_cast<float>(arguments));
		else if (param == "sparkPositionZ")
			explosionParameters->SetSparkPositionZ(lexical_cast<float>(arguments));

		else if (param == "fuelAreaRadiusInTexels")
			explosionParameters->SetFuelAreaRadiusInTexels(lexical_cast<int>(arguments));
		else if (param == "framesToDrawFuel")
			explosionParameters->SetFramesToDrawFuel(lexical_cast<int>(arguments));

		else if (param == "buoyancy")
			explosionParameters->SetBuoyancy(lexical_cast<float>(arguments));
		else if (param == "gravity")
			explosionParameters->SetGravity(lexical_cast<float>(arguments));

		else if (param == "burningRate")
			explosionParameters->SetBurningRate(lexical_cast<float>(arguments));
		else if (param == "stoichiometricMixture")
			explosionParameters->SetStoichiometricMixture(lexical_cast<float>(arguments));
		else if (param == "outputHeat")
			explosionParameters->SetOutputHeat(lexical_cast<float>(arguments));

		else if (param == "fuelDissipation")
			explosionParameters->SetFuelDissipation(lexical_cast<float>(arguments));
		else if (param == "sootDissipation")
			explosionParameters->SetSootDissipation(lexical_cast<float>(arguments));
		else if (param == "tempDissipation")
			explosionParameters->SetTempDissipation(lexical_cast<float>(arguments));

		else if (param == "fuelDiffusion")
			explosionParameters->SetFuelDiffusion(lexical_cast<float>(arguments));
		else if (param == "sootDiffusion")
			explosionParameters->SetSootDiffusion(lexical_cast<float>(arguments));
		else if (param == "tempDiffusion")
			explosionParameters->SetTempDiffusion(lexical_cast<float>(arguments));

		else if (param == "thermalExpansionRate")
			explosionParameters->SetThermalExpansionRate(lexical_cast<float>(arguments));
		else if (param == "vorticityConfinementStrength")
			explosionParameters->SetVorticityConfinementStrength(lexical_cast<float>(arguments));

		else if (param == "timeStep")
			explosionParameters->SetTimeStep(lexical_cast<float>(arguments));

		else if (param == "densityDiffusionJacobiIterations")
			explosionParameters->SetDensityDiffusionJacobiIterations(lexical_cast<int>(arguments));
		else if (param == "pressureGradientJacobiIterations")
			explosionParameters->SetPressureGradientJacobiIterations(lexical_cast<int>(arguments));
		
		else if (param == "simulateFreely")
			explosionParameters->SetSimulateFreely(lexical_cast<bool>(arguments));
		else if (param == "simulate3D")
			explosionParameters->SetSimulate3D(lexical_cast<bool>(arguments));
		else if (param == "renderDebuggedParticles")
			explosionParameters->SetRenderDebuggedParticles(lexical_cast<bool>(arguments));


		else if (param == "smoke.particleMass.lightPercentage")
			explosionParameters->smokeParticleMassLightPercentage = lexical_cast<float>(arguments);
		else if (param == "smoke.particleMass.lightBeginValue")
			explosionParameters->smokeParticleMassLightBeginValue = lexical_cast<float>(arguments);
		else if (param == "smoke.particleMass.lightEndValue")
			explosionParameters->smokeParticleMassLightEndValue = lexical_cast<float>(arguments);
		else if (param == "smoke.particleMass.heavyBeginValue")
			explosionParameters->smokeParticleMassHeavyBeginValue = lexical_cast<float>(arguments);
		else if (param == "smoke.particleMass.heavyEndValue")
			explosionParameters->smokeParticleMassHeavyEndValue = lexical_cast<float>(arguments);
		

		else if (param == "fire.particleMass.lightPercentage")
			explosionParameters->fireParticleMassLightPercentage = lexical_cast<float>(arguments);
		else if (param == "fire.particleMass.lightBeginValue")
			explosionParameters->fireParticleMassLightBeginValue = lexical_cast<float>(arguments);
		else if (param == "fire.particleMass.lightEndValue")
			explosionParameters->fireParticleMassLightEndValue = lexical_cast<float>(arguments);
		else if (param == "fire.particleMass.heavyBeginValue")
			explosionParameters->fireParticleMassHeavyBeginValue = lexical_cast<float>(arguments);
		else if (param == "fire.particleMass.heavyEndValue")
			explosionParameters->fireParticleMassHeavyEndValue = lexical_cast<float>(arguments);

		else if (param == "vortexParticle.count")
			explosionParameters->vortexParticleCount = lexical_cast<int>(arguments);
		else if (param == "vortexParticle.maxStrength")
			explosionParameters->vortexParticleMaxStrength = lexical_cast<float>(arguments);
		else if (param == "vortexParticle.maxRadius")
			explosionParameters->vortexParticleMaxRadius = lexical_cast<float>(arguments);

		else if (param == "particles.initialAreaRadius")
			explosionParameters->initialParticleAreaRadius = lexical_cast<float>(arguments);


		else
		{
			OutputDebugStringA("Unknown parameter in config file.\r\n");		
		}
	}
}
