#include "stdafx.h"

#include "MainFrame.h"

#include "CameraComponent.h"
#include "ClearBuffersComponent.h"
#include "ExplosionParameters.h"
#include "ExplosionSimulator2d.h"
#include "ExplosionSimulator3d.h"
#include "FpsComponent.h"
#include "ObstacleVisualizer.h"
#include "ParticleVisualizer.h"
#include "TextureStore.h"

#include <string>
#include <fstream>
#include <iostream>

bool isFilename(std::string &line)
{
    return line.size() > 0 && !(line[0] == line[1] && line[0] == '/');
}

std::string ReadFilenameFromFile(std::string filename)
{
    std::ifstream file;
    file.open(filename.c_str());

    std::string result = "";
    if (file) {
        std::string line;

        while (std::getline(file, line)) {
            if (isFilename(line)) {
                result = line;
                break;
            }
        }

    } else {
        assert(!"Could not find configuration file");
    }

    file.close();
    return result;
}

bool MainFrameInitializer::InitializeMainFrame(MainFrame &mainFrame)
{

    std::string filename = ReadFilenameFromFile("paramFileSpecifier.txt");
    if (filename.size() > 0) {
        ExplosionParameters::Instance().InitFromFile(filename);
    }

    CameraComponent* cameraComponent = new CameraComponent();
    FpsComponent* fpsComponent = new FpsComponent();

    int order = 0;

    fpsComponent->SetDrawOrder(order++);
    cameraComponent->SetUpdateOrder(order++);

    mainFrame.AddComponent(fpsComponent);
    mainFrame.AddComponent(cameraComponent);
    mainFrame.AddMessageHandler(cameraComponent);

    if (ExplosionParameters::Instance().ShouldSimulate3D()) {
        ExplosionSimulator3d* explosionSimulator = new ExplosionSimulator3d();
        ClearBuffersComponent* clearBuffersComponent = new ClearBuffersComponent();
        ObstacleVisualizer* obstacleVisualizer = new ObstacleVisualizer(explosionSimulator, cameraComponent);
        ParticleVisualizer* fireParticleVisualizer = new ParticleVisualizer(Fire, explosionSimulator, cameraComponent);
        ParticleVisualizer* smokeParticleVisualizer = new ParticleVisualizer(Smoke, explosionSimulator, cameraComponent);

        explosionSimulator->SetDrawOrder(order++);
        clearBuffersComponent->SetDrawOrder(order++);
        obstacleVisualizer->SetDrawOrder(order++);
        fireParticleVisualizer->SetDrawOrder(order++);
        smokeParticleVisualizer->SetDrawOrder(order++);

        mainFrame.AddComponent(explosionSimulator);
        mainFrame.AddComponent(clearBuffersComponent);
        mainFrame.AddComponent(obstacleVisualizer);
        mainFrame.AddComponent(fireParticleVisualizer);
        mainFrame.AddComponent(smokeParticleVisualizer);

        mainFrame.AddKeyHandler(explosionSimulator);
    }else {
        ExplosionSimulator2d* explosionSimulator = new ExplosionSimulator2d();
        ClearBuffersComponent* clearBuffersComponent = new ClearBuffersComponent();
        ParticleVisualizer* fireParticleVisualizer = new ParticleVisualizer(Fire, explosionSimulator, cameraComponent);
        ParticleVisualizer* smokeParticleVisualizer = new ParticleVisualizer(Smoke, explosionSimulator, cameraComponent);

        explosionSimulator->SetDrawOrder(order++);
        clearBuffersComponent->SetDrawOrder(order++);
        fireParticleVisualizer->SetDrawOrder(order++);
        smokeParticleVisualizer->SetDrawOrder(order++);

        mainFrame.AddComponent(explosionSimulator);
        mainFrame.AddComponent(clearBuffersComponent);
        mainFrame.AddComponent(fireParticleVisualizer);
        mainFrame.AddComponent(smokeParticleVisualizer);

        mainFrame.AddKeyHandler(explosionSimulator);
    }

    return true;
}
