#include "stdafx.h"

#include "CalculateDensityChangesOperation2d.h"
#include "EffectManager.h"
#include "ExplosionParameters.h"

CalculateDensityChangesOperation2d::CalculateDensityChangesOperation2d(ID3D10Device* device, TextureStore* textureStore,
                                                                       int densityId, int densitySourcesId)
    :
    effectName("ExplosionSimulator2d.fx"),
    textureStore(textureStore),
    densityId(densityId),
    densitySourcesId(densitySourcesId)
{
    InitOperation(device);
}

CalculateDensityChangesOperation2d::~CalculateDensityChangesOperation2d()
{
}

void CalculateDensityChangesOperation2d::InitOperation(ID3D10Device* device)
{
    ID3D10Effect* effect = EffectManager::Instance().GetEffect(effectName, device);

    calculateDensitySourcesTechnique = auto_ptr<TechniqueOperation>(
        new TechniqueOperation(effect, "CalculateDensitySourcesTechnique", textureStore, device)
    );
    calculateDensitySourcesOperation = auto_ptr<HlslTextureOperation2d>(
        new HlslTextureOperation2d(
            device,
            ExplosionParameters::Instance().GetSimulationWidth(),
            ExplosionParameters::Instance().GetSimulationHeight(),
            calculateDensitySourcesTechnique.get(),
            calculateDensitySourcesTechnique.get(),
            USE_DISPLACED_BORDER
        )
    );

    calculateDensitySourcesTechnique->AddSetShaderResourceTask("density", densityId);

    calculateDensitySourcesOperation->SetOutputTextureID(densitySourcesId);
}

void CalculateDensityChangesOperation2d::Run(float timestep, ID3D10Device* device)
{
    calculateDensitySourcesOperation->Execute(device);
}
