#include "stdafx.h"

#include "CalculateThermalExpansionOperation2d.h"
#include "EffectManager.h"
#include "ExplosionParameters.h"

CalculateThermalExpansionOperation2d::CalculateThermalExpansionOperation2d(ID3D10Device* device, TextureStore* textureStore,
                                                                           int densitySourcesId, int thermalExpansionId)
    :
    effectName("ExplosionSimulator2d.fx"),
    textureStore(textureStore),
    densitySourcesId(densitySourcesId),
    thermalExpansionId(thermalExpansionId)
{
    InitOperation(device);
}

CalculateThermalExpansionOperation2d::~CalculateThermalExpansionOperation2d()
{
}

void CalculateThermalExpansionOperation2d::InitOperation(ID3D10Device* device)
{
    ID3D10Effect* effect = EffectManager::Instance().GetEffect(effectName, device);

    calculateThermalExpansionTechnique = auto_ptr<TechniqueOperation>(
        new TechniqueOperation(effect, "CalculateThermalExpansionTechnique", textureStore, device)
    );
    calculateThermalExpansionOperation = auto_ptr<HlslTextureOperation2d>(
        new HlslTextureOperation2d(
            device,
            ExplosionParameters::Instance().GetSimulationWidth(),
            ExplosionParameters::Instance().GetSimulationHeight(),
            calculateThermalExpansionTechnique.get(),
            calculateThermalExpansionTechnique.get(),
            USE_DISPLACED_BORDER
        )
    );

    calculateThermalExpansionTechnique->AddSetShaderResourceTask("densitySources", densitySourcesId);

    calculateThermalExpansionOperation->SetOutputTextureID(thermalExpansionId);
}

void CalculateThermalExpansionOperation2d::Run(float timestep, ID3D10Device* device)
{
    calculateThermalExpansionOperation->Execute(device);
}
