#pragma once

#include "CalculateDensityChangesOperation2d.h"
#include "CalculateThermalExpansionOperation2d.h"
#include "CalculateVelocityForcesOperation2d.h"
#include "DensityOperation2d.h"
#include "DrawableGameComponent.h"
#include "DXUT.h"
#include "IKeyHandler.h"
#include "ISimulationResultProvider.h"
#include "TextureStore.h"
#include "VelocityOperation2d.h"

class ExplosionSimulator2d : public DrawableGameComponent, 
                             public ISimulationResultProvider, 
                             public IKeyHandler
{
public:
    ExplosionSimulator2d();

    virtual void Update(double time, float elapsedTime);
    virtual void Draw(ID3D10Device* device, double time, float elapsedTime);
    virtual bool LoadGeneralGraphicsContent(ID3D10Device* device, const DXGI_SURFACE_DESC *bufferSurfaceDesc);
    virtual void ReleaseGeneralGraphicsContent();

    TextureStore* GetTextureStore();
    int GetDensityResultID();
    int GetVelocityResultID();
    int GetObstaclesID();

    void HandleKeyEvent(UINT nChar, bool keyDown, bool altDown);

    bool ShouldSimulate();

private:
    void InitializeEffectVariables(ID3D10Device* device);
    bool InitializeTextures(ID3D10Device* device);
    bool InitializeSimulationOperations(ID3D10Device* device);
    bool InitializeCommonEffectVariables(ID3D10Device* device);

    void InitializeFuel(ID3D10Device* device);
    void InitializeSpark(ID3D10Device* device);

    void CleanUp() {}

private:
    CalculateDensityChangesOperation2d* calculateDensityChangesOperation;
    CalculateVelocityForcesOperation2d* calculateVelocityForcesOperation;
    CalculateThermalExpansionOperation2d* calculateThermalExpansionOperation;
    VelocityOperation2d* velocityOperation;
    DensityOperation2d* densityOperation;

    TextureStore* textureStore;

    int densityId;
    int velocityId;
    int thermalExpansionId;
    int densitySourcesId;
    int velocityForcesId;

    int framesToSimulate;
    bool simulateFreely;
};
