#pragma once

#include "DrawableGameComponent.h"
#include "ISimulationResultProvider.h"
#include "DXUT.h"
#include "IKeyHandler.h"

class TextureStore;
class ExplosionSpecificOperation3d;
class VelocityOperation3d;
class DensityOperation3d;
class VortexParticleOperation3d;
class TechniqueOperation;

class ExplosionSimulator3d : public DrawableGameComponent, public ISimulationResultProvider, public IKeyHandler
{
public:

    static ExplosionSimulator3d* Instance;

    ExplosionSimulator3d();

    virtual void Update(double time, float elapsedTime);
    virtual void Draw(ID3D10Device* device, double time, float elapsedTime);
    virtual bool LoadGeneralGraphicsContent(ID3D10Device* device, const DXGI_SURFACE_DESC *bufferSurfaceDesc);
    virtual void ReleaseGeneralGraphicsContent();

    TextureStore* GetTextureStore();
    int GetDensityResultID();
    int GetVelocityResultID();
    int GetObstaclesID();

    void HandleKeyEvent(UINT nChar, bool keyDown, bool altDown);

    bool ShouldSimulate();

private:
    bool InitTextures(ID3D10Device* device);
    bool InitSimulationOperations(ID3D10Device* device);
    bool InitCommonEffectVariables(ID3D10Device* device);
    void InitializeObstacles(ID3D10Device* device);

    void InitializeDensities(ID3D10Device* device);
    void InitializeFuel(ID3D10Device* device);
    void InitializeSpark(ID3D10Device* device);

    void DrawCube(ID3D10Device* device,
                    float xMin, float xMax,
                    float yMin, float yMax,
                    int zMin, int zMax,
                    TechniqueOperation* technique);

    void CleanUp() {}
private:
    VortexParticleOperation3d* vortexParticleOperation;
    ExplosionSpecificOperation3d* explosionSpecificOperation;
    VelocityOperation3d* velocityOperation;
    DensityOperation3d* densityOperation;

    TextureStore* textureStore;

    int densityId;
    int velocityId;
    int thermalExpansionId;
    int obstaclesId;

    int framesToSimulate;
    bool simulateFreely;

public:
    int zSliceToDraw;
};
