#include "stdafx.h"

#include "ExplosionSpecificOperation3d.h"
#include "TextureStore.h"
#include "EffectManager.h"
#include "HlslTextureOperation3d.h"
#include "TechniqueOperation.h"
#include "ExplosionParameters.h"

ExplosionSpecificOperation3d::ExplosionSpecificOperation3d(ID3D10Device* device, TextureStore* textureStore,
                                                       int densityID, int velocityID, int thermalExpansionID, int obstaclesID )
                                                       : textureStore(textureStore),
                                                       densityID(densityID),
                                                       velocityID(velocityID),
                                                       obstaclesID(obstaclesID),
                                                       thermalExpansionID(thermalExpansionID),
                                                       effectName("ExplosionSimulator3d.fx")
{
    InitVorticityOperation(device);
    InitExplosionSpecificOperation(device);
    InitEffectVariables(device);
}

void ExplosionSpecificOperation3d::InitVorticityOperation(ID3D10Device* device)
{
    vorticityID = textureStore->AddTexture("Vorticity", device);

    ID3D10Effect* effect = EffectManager::Instance().GetEffect(effectName, device);
    calculateVorticityTechnique = auto_ptr<TechniqueOperation>(
                                    new TechniqueOperation(effect, "CalculateVorticityTechnique", textureStore, device));
    calculateVorticityOperation = auto_ptr<HlslTextureOperation3d>(new HlslTextureOperation3d(device,
        ExplosionParameters::Instance().GetSimulationWidth(),
        ExplosionParameters::Instance().GetSimulationHeight(),
        ExplosionParameters::Instance().GetSimulationDepth(),
        calculateVorticityTechnique.get(), calculateVorticityTechnique.get()));

    calculateVorticityTechnique->AddSetShaderResourceTask("velocity", velocityID);
    calculateVorticityOperation->SetOutputTextureID(vorticityID);
}

void ExplosionSpecificOperation3d::InitExplosionSpecificOperation( ID3D10Device* device )
{
    ID3D10Effect* effect = EffectManager::Instance().GetEffect(effectName, device);
    explosionTechnique = auto_ptr<TechniqueOperation>(
                                    new TechniqueOperation(effect, "ExplosionSpecificsTechnique", textureStore, device));

    explosionOperation = auto_ptr<HlslTextureOperation3d>(new HlslTextureOperation3d(device,
        ExplosionParameters::Instance().GetSimulationWidth(),
        ExplosionParameters::Instance().GetSimulationHeight(),
        ExplosionParameters::Instance().GetSimulationDepth(),
        explosionTechnique.get(), explosionTechnique.get()));

    explosionTechnique->AddSetShaderResourceTask("density", densityID);
    explosionTechnique->AddSetShaderResourceTask("velocity", velocityID);
    explosionTechnique->AddSetShaderResourceTask("vorticity", vorticityID);
    explosionTechnique->AddSetShaderResourceTask("obstacles", obstaclesID);

    explosionOperation->SetOutputTextureIDs(densityID, velocityID, thermalExpansionID);
}

void ExplosionSpecificOperation3d::InitEffectVariables(ID3D10Device* device)
{
    ID3D10Effect* effect = EffectManager::Instance().GetEffect(effectName, device);
    ExplosionParameters& p = ExplosionParameters::Instance();

    effect->GetVariableByName("LowerIgnitionLimit")->AsScalar()->SetFloat(p.GetLowerIgnitionLimit());
    effect->GetVariableByName("BurningRate")->AsScalar()->SetFloat(p.GetBurningRate());
    effect->GetVariableByName("StoichiometricMixture")->AsScalar()->SetFloat(p.GetStoichiometricMixture());
    effect->GetVariableByName("OutputHeat")->AsScalar()->SetFloat(p.GetOutputHeat());

    effect->GetVariableByName("ThermalExpansionRate")->AsScalar()->SetFloat(p.GetThermalExpansionRate());

    effect->GetVariableByName("Buoyancy")->AsScalar()->SetFloat(p.GetBuoyancy());
    effect->GetVariableByName("Gravity")->AsScalar()->SetFloat(p.GetGravity());

    effect->GetVariableByName("VorticityConfinementStrength")->AsScalar()->SetFloat(p.GetVorticityConfinementStrength());

    effect->GetVariableByName("Dissipation")->AsVector()->SetFloatVector(p.GetDissipation());
}

ExplosionSpecificOperation3d::~ExplosionSpecificOperation3d()
{
}

void ExplosionSpecificOperation3d::Run(float timestep, ID3D10Device* device)
{
    calculateVorticityOperation->Execute(device);
    explosionOperation->Execute(device);
}
