#include "stdafx.h"

#include "BlackbodyRadiationManager.h"

const double BlackbodyRadiationManager::h = 6.626e-34;
const double BlackbodyRadiationManager::c = 3.0e8;
const double BlackbodyRadiationManager::k = 1.38e-23;

const double BlackbodyRadiationManager::redWave = 0.7e-6;
const double BlackbodyRadiationManager::greenWave = 0.56e-6;
const double BlackbodyRadiationManager::blueWave = 0.47e-6;

const double BlackbodyRadiationManager::exposureLevel = 16;

const BlackbodyRadiationManager& BlackbodyRadiationManager::Instance()
{
	static BlackbodyRadiationManager instance;
	return instance;
}

const D3DXVECTOR4& BlackbodyRadiationManager::GetColor(int level) const
{
	assert(level >= 0 && level < 256 && "Level has to be in the range [0, 256>");
	return colors[level];
}

const D3DXVECTOR4* BlackbodyRadiationManager::GetAllColors() const
{
	return colors;
}

BlackbodyRadiationManager::BlackbodyRadiationManager()
{
	Initialize();
}

BlackbodyRadiationManager::~BlackbodyRadiationManager()
{}

void BlackbodyRadiationManager::Initialize()
{
	for (int i = 0; i < 256; ++i) {
		double T = 1240 + 5.2f * i;

		int red = (int) Expose(LightPower(redWave, T), exposureLevel);
		int green = (int) Expose(LightPower(greenWave, T), exposureLevel);
		int blue = (int) Expose(LightPower(blueWave, T), exposureLevel);

		colors[i].x = red / 256.0f;
		colors[i].y = green / 256.0f;
		colors[i].z = blue / 256.0f;
		colors[i].w = 1.0f;
	}
}

double BlackbodyRadiationManager::Expose(double light, double exposure)
{
	return (1.0 - exp(-light * exposure)) * 255;
}

double BlackbodyRadiationManager::LightPower(double waveLength, double temperature)
{
	double L = waveLength;
	double T = temperature;
	return 10.0e-27 / (pow(L, 5) * (exp((h * c) / (k * L * T)) - 1.0));
}
