#pragma once

#include "DrawableGameComponent.h"
#include "DXUT.h"
#include "ISimulationResultProvider.h"

#pragma warning (disable: 4995)
#include <string>

using namespace std;

class TextureStore;
class TechniqueOperation;
class ICameraProvider;
class VertexPositionVelocityRMass;

enum ParticleSystemType
{
	Fire,
	Smoke
};

class ParticleVisualizer : public DrawableGameComponent
{
public:
	ParticleVisualizer(ParticleSystemType type, ISimulationResultProvider* simulator, ICameraProvider* cameraProvider);
	virtual void Draw(ID3D10Device* device, double time, float elapsedTime);
	virtual bool LoadGeneralGraphicsContent(ID3D10Device* device, const DXGI_SURFACE_DESC *bufferSurfaceDesc);
	virtual void ReleaseGeneralGraphicsContent();

private:
	void InitEffectVariables(ID3D10Device* device);
	void InitAdvanceParticlesTechnique(ID3D10Device* device);
	void InitRenderParticlesTechnique(ID3D10Device* device);
	void InitBlackBodyRadiationTable(ID3D10Device* device);
	void InitRandomValuesTable(ID3D10Device* device);
	void InitTextures(ID3D10Device* device);
	void InitKolmogorovTurbulenceTextures(ID3D10Device* device);

	void InitBuffers(ID3D10Device* device);
	void InitRandomParticlePositions(VertexPositionVelocityRMass* particles, int particleCount);

	void HandleTurbulenceInterpolation();
	void AdvanceParticles(ID3D10Device* device, double time);
	void RenderParticles(ID3D10Device* device);

private:
	ISimulationResultProvider* simulator;
	ID3D10InputLayout* inputLayout;

	TechniqueOperation* advanceParticlesTechnique;
	TechniqueOperation* renderParticlesTechnique;

	ID3D10Buffer* drawFromBuffer;
	ID3D10Buffer* streamOutToBuffer;

	int renderToTextureID;
	int colorLookupTextureID;

    D3D10_VIEWPORT vp;
	ICameraProvider* cameraProvider;

	ID3D10EffectMatrixVariable* worldViewProjectionMatrixVariable;
	ID3D10EffectScalarVariable* timeVariable;

	ID3D10ShaderResourceView* volumeTextureView;
	ID3D10ShaderResourceView* fooTextureView;

	ID3D10ShaderResourceView* turbulenceVolumeView1;
	ID3D10ShaderResourceView* turbulenceVolumeView2;
	ID3D10EffectScalarVariable* turbulenceInterpolationVariable;
	float turbulenceInterpolatorCounter;
	float globalTime;
	int frame;

protected:
	template <class T> void CreateBuffers(T * particles, ID3D10Device* device);

	ParticleSystemType particleSystemType;

	string effectFileName;
	string advanceParticlesTechniqueName;
	string renderParticlesTechniqueName;

	int initialParticleCount;
	int maxParticleCount;

	bool isInitialized;

	void InitStuff(ID3D10Device* device);
};


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Template definition																								   //	
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


template <class T> void ParticleVisualizer::CreateBuffers(T * particles, ID3D10Device* device)
{
	ID3D10Buffer* initialBuffer = VertexBufferUtils::CreateDefaultVertexBuffer(device, particles, initialParticleCount);

	this->drawFromBuffer = VertexBufferUtils::CreateSOVertexBuffer<T>(device, maxParticleCount);
	this->streamOutToBuffer = VertexBufferUtils::CreateSOVertexBuffer<T>(device, maxParticleCount);
	
	this->inputLayout = VertexBufferUtils::CreateInputLayout<T>
		(device, renderParticlesTechnique->GetFirstPass());

	VertexBufferUtils::SetSingleVertexBuffer<T>(device, initialBuffer);
	VertexBufferUtils::SetSingleSOVertexBuffer(device, drawFromBuffer);
	
	device->IASetInputLayout(inputLayout);
	device->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_POINTLIST);
	
	advanceParticlesTechnique->Execute(initialParticleCount, device);
	
	device->SOSetTargets(0,0,0);

	SAFE_RELEASE(initialBuffer);

}
