#pragma once

#include "ISimulationOperation.h"

#include "DXUT.h"

#pragma warning (disable:4995)
#include <memory>
#include <string>

class TextureStore;
class TechniqueOperation;
class HlslTextureOperation;

using namespace std;

class DensityOperation2d : public ISimulationOperation
{
public:
	DensityOperation2d(ID3D10Device* device, TextureStore* textureStore, 
	                   int densityID, int velocityID,
	                   int densitySourcesID);
	~DensityOperation2d();
	void Run(float timestep, ID3D10Device* device);

	void SetJacobiIterations(int iterationCount);

private:
    void InitAddDensitySourcesOperation(ID3D10Device* device);
	void InitAdvectionOperation(ID3D10Device* device);
	void InitDiffusionOperation(ID3D10Device* device);
	void InitEffectVariables(ID3D10Device* device);

private:
    auto_ptr<TechniqueOperation> addDensitySourcesTechnique;
    auto_ptr<HlslTextureOperation> addDensitySourcesOperation;

	auto_ptr<TechniqueOperation> advectionTechnique;
	auto_ptr<HlslTextureOperation> advectionOperation;

	auto_ptr<TechniqueOperation> diffusionTechnique;
	auto_ptr<TechniqueOperation> borderTechnique;
	auto_ptr<HlslTextureOperation> diffusionOperation;

	string effectName;

	TextureStore* textureStore;
	int densityID;
	int densitySourcesID;
	int diffusedDensityID;
	int velocityID;

	int jacobiIterations;
};
