#pragma once
#include "ISimulationOperation.h"
#include "DXUT.h"

#pragma warning (disable:4995)
#include <memory>
#include <string>

class TextureStore;
class TechniqueOperation;
class HlslTextureOperation;

using namespace std;

class DensityOperation3d : public ISimulationOperation
{
public:
	DensityOperation3d(ID3D10Device* device, TextureStore* textureStore, int velocityID, int densityID, int obstaclesID);
	~DensityOperation3d();
	void Run(float timestep, ID3D10Device* device);

	void SetJacobiIterations(int iterationCount);

private:
	void InitAdvectionOperation(ID3D10Device* device);
	void InitDiffusionOperation(ID3D10Device* device);
	void InitEffectVariables(ID3D10Device* device);

	auto_ptr<TechniqueOperation> advectionTechnique;
	auto_ptr<HlslTextureOperation> advectionOperation;

	auto_ptr<TechniqueOperation> diffusionTechnique;
	auto_ptr<TechniqueOperation> borderTechnique;
	auto_ptr<HlslTextureOperation> diffusionOperation;

	string effectName;

	TextureStore* textureStore;
	int velocityID;
	int densityID;
	int diffusionSolutionGuessID;
	int obstaclesID;

	int jacobiIterations;
};
