#pragma once
#include "DXUT.h"
#include "ISimulationOperation.h"

#pragma warning (disable:4995)
#include <memory>
#include <string>

using namespace std;

class TextureStore;
class TechniqueOperation;
class HlslTextureOperation;

class VelocityOperation2d : public ISimulationOperation
{
public:
    VelocityOperation2d(ID3D10Device* device, TextureStore* textureStore, 
                        int velocityID, int divergenceForcesID, int velocityForcesID);
    ~VelocityOperation2d();
    virtual void Run(float timestep, ID3D10Device* device);

    void SetJacobiIterations(int iterationCount);

private:
    void InitAddVelocityForcesOperation(ID3D10Device* device);

    void InitAdvectionOperation(ID3D10Device* device);

    void InitCalculateDivergenceOperation(ID3D10Device* device);
    void InitCalculatePressureOperation(ID3D10Device* device);
    void InitSubtractGradientOperation(ID3D10Device* device);

    void InitEffectVariables(ID3D10Device* device);

    auto_ptr<TechniqueOperation> addVelocityForcesTechnique;
    auto_ptr<HlslTextureOperation> addVelocityForcesOperation;

    auto_ptr<TechniqueOperation> advectionTechnique;
    auto_ptr<TechniqueOperation> advectionBorderTechnique;
    auto_ptr<HlslTextureOperation> advectionOperation;

    auto_ptr<TechniqueOperation> calculateDivergenceTechnique;
    auto_ptr<HlslTextureOperation> calculateDivergenceOperation;

    auto_ptr<TechniqueOperation> calculatePressureTechnique;
    auto_ptr<TechniqueOperation> borderTechnique;
    auto_ptr<HlslTextureOperation> calculatePressureOperation;

    auto_ptr<TechniqueOperation> subtractGradientTechnique;
    auto_ptr<HlslTextureOperation> subtractGradientOperation;

    string effectName;

    TextureStore* textureStore;
    int velocityID;
    int velocityForcesID;
    int divergenceID;
    int divergenceForcesID;
    int pressureID;

    int jacobiIterations;
};
