#include "stdafx.h"

#include "EffectManager.h"
#include "SDKmisc.h"

EffectManager::EffectManager()
{
}

EffectManager::~EffectManager()
{
	ReleaseAll();
}

void EffectManager::ReleaseAll()
{
	map<string, ID3D10Effect*>::iterator it;
	it = effectMap.begin();
	map<string, ID3D10Effect*>::iterator end = effectMap.end();
	for (; it != end; ++it) {
		SAFE_RELEASE(it->second);
	}
	effectMap.clear();
}

ID3D10Effect* EffectManager::CreateEffectFromFile(string filename, ID3D10Device* device, D3D10_SHADER_MACRO *shader_Macros) {

	WCHAR buf[128];
	MultiByteToWideChar(CP_ACP, 0, filename.c_str(), -1, buf, 128);

	WCHAR str[512];
	if (DXUTFindDXSDKMediaFileCch( str, MAX_PATH, buf ) == DXUTERR_MEDIANOTFOUND) {
		MessageBox( NULL, L"Could not find fx file.", L"Error", MB_OK );
		return NULL;
	}
	ID3D10Effect* effect;
	HRESULT hr = D3DX10CreateEffectFromFile( str, shader_Macros, NULL, D3D10_SHADER_ENABLE_STRICTNESS, 0, device, NULL, NULL, &effect, NULL );
	if( FAILED( hr ) )
	{
		MessageBox( NULL, L"Compiling fx file failed.", L"Error", MB_OK );
		return NULL;
	}
	return effect;
}

void EffectManager::PrepareEffectWithMacros(string filename, ID3D10Device* device, D3D10_SHADER_MACRO *shader_Macros)
{
	if(effectMap.find(filename) == effectMap.end()) {
		ID3D10Effect* effect = CreateEffectFromFile(filename.c_str(), device, shader_Macros);
		if (effect != NULL) {
		effectMap[filename] = effect;				
		}
	}
}


ID3D10Effect* EffectManager::GetEffect(string filename, ID3D10Device* device) 
{
	ID3D10Effect* effect = NULL;
	if (effectMap.find(filename) != effectMap.end()) {
		effect = effectMap[filename];
	} else {
		effect = CreateEffectFromFile(filename.c_str(), device);
		if (effect != NULL) effectMap[filename] = effect;
	}
	return effect;
}

EffectManager& EffectManager::Instance()
{
	static EffectManager EffectManager;
	return EffectManager;
}
