#pragma once

#include "DXUT.h"
#include "HlslTextureOperation.h"
#include "TechniqueOperation.h"

#pragma warning (disable:4995)
#include <string>

using namespace std;

enum BorderOperation
{
	USE_REGULAR_BORDER,
	USE_DISPLACED_BORDER,
	IGNORE_BORDER
};

class HlslTextureOperation2d : public HlslTextureOperation
{
public:
	HlslTextureOperation2d(ID3D10Device* device, int width, int height, 
						 TechniqueOperation* interiorTechnique, TechniqueOperation* borderTechnique, 
						 BorderOperation borderOperation = USE_REGULAR_BORDER);
	~HlslTextureOperation2d();
	
	void Execute(ID3D10Device* device);

private:
	void InitInteriorVertexBuffer(ID3D10Device* device, BorderOperation borderOp);
	void InitBorderVertexBuffer(ID3D10Device* device, BorderOperation borderOperation);
	void ActivateInteriorBuffer(ID3D10Device* device);
	void ActivateBorderBuffer(ID3D10Device* device);
	void SetOutputTargets(ID3D10Device* device);
	void CheckRenderTargetSwapping();
	void ClearOutputTextures();

	int width;
	int height;

	TechniqueOperation* interiorTech;
	TechniqueOperation* borderTech;

	ID3D10Buffer* interiorVB;
	int interiorVertexCount;
	ID3D10InputLayout* interiorInputLayout;
	D3D10_PRIMITIVE_TOPOLOGY interiorPrimitiveTopology;

	ID3D10Buffer* borderVB;
	int borderVertexCount;
	ID3D10InputLayout* borderInputLayout;
	D3D10_PRIMITIVE_TOPOLOGY borderPrimitiveTopology;

	D3D10_VIEWPORT oldViewPort;

	static const int MAX_RENDER_TARGETS = 8;
	TextureStore* textureStore;
	bool hasTestedSwapping;
};
