#pragma once

#include "DXUT.h"
#include "HlslTextureOperation.h"
#include "TechniqueOperation.h"
#include "VertexDefinition.h"

class HlslTextureOperation3d : public HlslTextureOperation
{
public:
	HlslTextureOperation3d(ID3D10Device* device, int width, int height, int depth,
						 TechniqueOperation* interiorTechnique, TechniqueOperation* borderTechnique);
	~HlslTextureOperation3d();

	void Execute(ID3D10Device* device);

private:
	void InitInteriorVertexBuffer(ID3D10Device* device);
	void InitBorderVertexBuffer(ID3D10Device* device);

	void CreateInteriorSliceVertices(VertexPositionTexture3D* vertices, int startIndex, int z);
	void ActivateInteriorBuffer(ID3D10Device* device);

	void SetOutputTargets(ID3D10Device* device);
	void CheckRenderTargetSwapping();


private:
	int width;
	int height;
	int depth;

	TechniqueOperation* interiorTech;
	TechniqueOperation* borderTech;

	ID3D10Buffer* interiorVB;
	int interiorVertexCount;
	ID3D10InputLayout* interiorInputLayout;
	D3D10_PRIMITIVE_TOPOLOGY interiorPrimitiveTopology;

	ID3D10Buffer* borderVB;
	int borderVertexCount;
	ID3D10InputLayout* borderInputLayout;
	D3D10_PRIMITIVE_TOPOLOGY borderPrimitiveTopology;

	D3D10_VIEWPORT oldViewPort;

	static const int MAX_RENDER_TARGETS = 8;
	TextureStore* textureStore;
	vector<pair<int,bool> > renderTargetInfo;
	bool hasTestedSwapping;

};
