#include "stdafx.h"

#include "EffectUtils.h"
#include "HlslTextureOperation2d.h"
#include "TechniqueOperation.h"

#include <sstream>

using namespace std;

TechniqueOperation::~TechniqueOperation()
{
}

TechniqueOperation::TechniqueOperation(ID3D10Effect* effect, string techniqueName, 
									   TextureStore* textureStore, ID3D10Device* device)
: 
effect(NULL), technique(NULL), textureStore(textureStore)
{
	this->effect = effect;
	if (!effect || !effect->IsValid()) return;

	InitTechniques(techniqueName);	
}

void TechniqueOperation::InitTechniques(string techniqueName)
{
	technique = effect->GetTechniqueByName(techniqueName.c_str());
	if (!technique->IsValid()) {
		stringstream ss;
		ss << "Cannot find technique: " << techniqueName;
		string s = ss.str();
		throw std::exception(s.c_str());
	}
}

void TechniqueOperation::ExecuteAuto(ID3D10Device* device)
{
	SetShaderResources();
	int numPasses = EffectUtils::GetNumberOfTechniquePasses(technique);
	for (int p = 0; p < numPasses; ++p)
	{
		technique->GetPassByIndex(p)->Apply(0);
		device->DrawAuto();
	}
	RemoveShaderResources();
}

void TechniqueOperation::Execute(unsigned int vertexCount, ID3D10Device* device)
{
	SetShaderResources();
	int numPasses = EffectUtils::GetNumberOfTechniquePasses(technique);
	for (int p = 0; p < numPasses; ++p)
	{
		technique->GetPassByIndex(p)->Apply(0);
		device->Draw(vertexCount, 0);
	}
	RemoveShaderResources();
}

bool TechniqueOperation::IsValid()
{
	return this->effect    && this->effect->IsValid()    != 0  && 
		   this->technique && this->technique->IsValid() != 0;	
}

void TechniqueOperation::SetTextureStore(TextureStore* store)
{
	this->textureStore = store;
}

void TechniqueOperation::AddSetShaderResourceTask(string resourceName, int textureID)
{
	this->setShaderResourceTasks.push_back(pair<string, int>(resourceName, textureID));
}

bool TechniqueOperation::IsIDShaderResource(int textureID)
{
	for (unsigned int i = 0; i < setShaderResourceTasks.size(); ++i) {
		if (setShaderResourceTasks[i].second == textureID) return true;
	}
	return false;
}

void TechniqueOperation::SetShaderResources()
{
	for (unsigned int i = 0; i < setShaderResourceTasks.size(); ++i) {
		pair<string, int> task = setShaderResourceTasks[i];
		ID3D10EffectShaderResourceVariable* resource = this->effect->GetVariableByName(task.first.c_str())->AsShaderResource();
		textureStore->SetAsShaderResource(task.second, resource);
	}
}

void TechniqueOperation::RemoveShaderResources()
{
	for (unsigned int i = 0; i < setShaderResourceTasks.size(); ++i) {
		pair<string, int> task = setShaderResourceTasks[i];
		ID3D10EffectShaderResourceVariable* resource = this->effect->GetVariableByName(task.first.c_str())->AsShaderResource();
		resource->SetResource(NULL);
	}
	technique->GetPassByIndex(0)->Apply(0);
}

ID3D10EffectPass* TechniqueOperation::GetFirstPass()
{
	return technique->GetPassByIndex(0);
}

void TechniqueOperation::ApplyTechnique()
{
	technique->GetPassByIndex(0)->Apply(0);
}

TextureStore* TechniqueOperation::GetTextureStore()
{
	return textureStore;
}
