#pragma once

#include "DXUT.h"
#include "TextureStore.h"

#pragma warning (disable:4995)
#include <string>
#include <map>
#include <vector>

class HlslTextureOperation;
class TextureStore;

class TechniqueOperation
{
public:
	TechniqueOperation(ID3D10Effect* effect, string techniqueName, TextureStore* textureStore, ID3D10Device* device);
	~TechniqueOperation();

	bool IsValid();

	void ApplyTechnique();

	void ExecuteAuto(ID3D10Device* device);
	void Execute(unsigned int vertexCount, ID3D10Device* device);

	void SetTextureStore(TextureStore* store);
	void AddSetShaderResourceTask(string resourceName, int textureID);
	bool IsIDShaderResource(int textureID);
	ID3D10EffectPass* GetFirstPass();
	TextureStore* GetTextureStore();

private:
	void InitTechniques(string techniqueName);
	void InitEffectVariables();
	void SetShaderResources();
	void RemoveShaderResources();

	std::map< std::string, ID3D10EffectVariable*> effectVariables;
	std::map< std::string, ID3D10EffectTechnique*> techniques;
	ID3D10Effect* effect;
	ID3D10EffectTechnique* technique;
	TextureStore* textureStore;
	std::vector< std::pair<std::string, int> > setShaderResourceTasks;
};
