#pragma once
#include "DXUT.h"


#pragma warning (disable:4995)
#include <vector>
#include <list>
#include <string>
#include <exception>

using namespace std;

class TextureStore
{
public:
	TextureStore(int width, int height, ID3D10Device* device, int numberOfTempsNeeded = 1);
	virtual ~TextureStore();

	int AddTexture1D(string description, ID3D10Device* device, int width, const D3DXVECTOR4* initialData);
	virtual int AddTexture(string description, ID3D10Device* device) = 0;

	ID3D10Texture1D* GetTexture1D(int id);
	ID3D10Resource* GetResource(int id);

	ID3D10ShaderResourceView* GetShaderResourceView1D(int id);
	ID3D10ShaderResourceView* GetShaderResourceView(int id);

	void SetAsShaderResource1D(int id, ID3D10EffectShaderResourceVariable* variable);
	void SetAsShaderResource(int id, ID3D10EffectShaderResourceVariable* variable);

	void SetAsRenderTarget(int id, bool isSwapNeeded, ID3D10Device* device);
	void SetAsRenderTargets(const vector< pair<int, bool> >& targets, ID3D10Device* device);
	void Swap();
	void ClearRenderTarget(int id, const float RGBA[], ID3D10Device* device);
	void ZeroTexture(int id, ID3D10Device* device);

	void ManualSwap(int id1, int id2);

	virtual ID3D10Resource* GetStagingTexture(ID3D10Device* device) = 0;

protected:
	void InitTempTextures(int numberOfTempsNeeded, ID3D10Device* device);
	void InitializeViewPort();

	typedef struct {
		ID3D10Texture1D* texture;
		ID3D10ShaderResourceView* shaderResourceView;
		ID3D10RenderTargetView* renderTargetView;
		string desc;
	} Texture1DRefs;

	typedef struct {
		ID3D10Resource* texture;
		ID3D10ShaderResourceView* shaderResourceView;
		ID3D10RenderTargetView* renderTargetView;
		string desc;
	} ResourceRefs;

	vector<Texture1DRefs> textures1d;
	vector<ResourceRefs> resources;

	int width;
	int height;
	
	list<int> availableTemps;
	list<pair<int,int> > tempsInUseAs;

	D3D10_VIEWPORT viewPort;

	//ID3D10Texture2D* stagingTexture;
};
