#include "stdafx.h"

#include "TextureStore2D.h"
#include "TextureUtils.h"

#include <sstream>

TextureStore2D::TextureStore2D( int width, int height, ID3D10Device* device, int numberOfTempsNeeded )
	: 
	TextureStore(width, height, device, numberOfTempsNeeded), 
	stagingTexture(NULL)
{
	InitTempTextures(numberOfTempsNeeded, device);
}

TextureStore2D::~TextureStore2D()
{
	SAFE_RELEASE(stagingTexture);
}

int TextureStore2D::AddTexture(string description, ID3D10Device* device)
{
	ResourceRefs refs;
	{
		refs.desc = description;
		refs.texture = TextureUtils::CreateSimpleTexture2D(
			device, this->width, this->height, D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE
		);
	}

	device->CreateShaderResourceView(refs.texture, NULL, &refs.shaderResourceView);
	device->CreateRenderTargetView(refs.texture, NULL, &refs.renderTargetView);

	this->resources.push_back(refs);
	return (int)(resources.size()-1);
}

ID3D10Resource* TextureStore2D::GetStagingTexture(ID3D10Device* device) 
{
	if (stagingTexture == NULL) {
		stagingTexture = TextureUtils::CreateSimpleTexture2D(
			device, 
			width, 
			height, 
			0, D3D10_USAGE_STAGING, D3D10_CPU_ACCESS_READ);	
	}
	return stagingTexture;
}
