#include "stdafx.h"

#include "TextureStore3D.h"
#include "TextureUtils.h"

#include <sstream>

TextureStore3D::TextureStore3D( int width, int height, int depth, ID3D10Device* device, int numberOfTempsNeeded )
	: 
	TextureStore(width, height, device, numberOfTempsNeeded), 
	depth(depth), 
	stagingTexture(NULL)
{
	InitTempTextures(numberOfTempsNeeded, device);
}

TextureStore3D::~TextureStore3D()
{
	SAFE_RELEASE(stagingTexture);
}

int TextureStore3D::AddTexture(string description, ID3D10Device* device)
{
	ResourceRefs refs;
	{
		refs.desc = description;
		refs.texture = TextureUtils::CreateSimpleTexture3D(
			device, this->width, this->height, this->depth, D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE
		);
	}

	device->CreateShaderResourceView(refs.texture, NULL, &refs.shaderResourceView);
	device->CreateRenderTargetView(refs.texture, NULL, &refs.renderTargetView);

	this->resources.push_back(refs);
	return (int)(resources.size()-1);
}


ID3D10Resource* TextureStore3D::GetStagingTexture(ID3D10Device* device) 
{
	if (stagingTexture == NULL) {
		stagingTexture = TextureUtils::CreateSimpleTexture3D(
			device, 
			width, 
			height,
			depth,
			0, D3D10_USAGE_STAGING, D3D10_CPU_ACCESS_READ);	
	}
	return stagingTexture;
}
