#include "stdafx.h"

#include "TextureUtils.h"
#include <string>

TextureUtils::TextureUtils()
{
}

TextureUtils::~TextureUtils()
{
}

ID3D10Texture1D* TextureUtils::CreateTexture1D(ID3D10Device* device, 
											   int width, 
											   UINT bindFlags, 
											   D3D10_USAGE usage,
											   UINT cpuAccessFlags)
{
	return CreateTexture1D(device, width, NULL, bindFlags, usage, cpuAccessFlags);
}

ID3D10Texture1D* TextureUtils::CreateTexture1D(ID3D10Device* device, 
											   int width,
											   const D3DXVECTOR4* initialData,
											   UINT bindFlags, 
											   D3D10_USAGE usage,
											   UINT cpuAccessFlags)
{
	D3D10_TEXTURE1D_DESC desc;
	{
		ZeroMemory(&desc, sizeof(desc));
		desc.ArraySize = 1;
		desc.BindFlags = bindFlags;
		desc.CPUAccessFlags = cpuAccessFlags;
		desc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
		desc.MipLevels = 1;
		desc.Usage = usage;
		desc.Width = width;
	}

	HRESULT hr;

	ID3D10Texture1D* texture;
	{
		if (initialData != NULL) {
			D3D10_SUBRESOURCE_DATA subData;
			subData.pSysMem = (const void*) initialData;

			hr = device->CreateTexture1D(&desc, &subData, &texture);
		} else {
			hr = device->CreateTexture1D(&desc, NULL, &texture);
		}
	}

	if (FAILED(hr)) return NULL;
	else            return texture;
}

ID3D10Texture2D* TextureUtils::CreateSimpleTexture2D(ID3D10Device *device, int width, int height, UINT bindFlags, D3D10_USAGE usage, UINT cpuAccessFlags, DXGI_FORMAT format) 
{
	D3D10_TEXTURE2D_DESC desc;
	ZeroMemory(&desc, sizeof(desc));
	desc.Width = width;
	desc.Height = height;
	desc.MipLevels = 1;
	desc.ArraySize = 1;
	desc.Format = format;
	desc.SampleDesc.Count = 1;
	desc.SampleDesc.Quality = 0;
	desc.Usage = usage;
	desc.BindFlags = bindFlags;
	desc.CPUAccessFlags = cpuAccessFlags;
	
	ID3D10Texture2D* texture;
	HRESULT hr = device->CreateTexture2D(&desc, NULL, &texture);	
	
	if (FAILED(hr)) return NULL;
	else			return texture;
}

ID3D10Texture3D* TextureUtils::CreateSimpleTexture3D(ID3D10Device *device, int width, int height, int depth, UINT bindFlags, D3D10_USAGE usage, UINT cpuAccessFlags) 
{
	D3D10_TEXTURE3D_DESC desc;
	ZeroMemory(&desc, sizeof(desc));
	desc.Width = width;
	desc.Height = height;
	desc.Depth = depth;
	desc.MipLevels = 1;
	desc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	desc.Usage = usage;
	desc.BindFlags = bindFlags;
	desc.CPUAccessFlags = cpuAccessFlags;
	desc.MiscFlags = 0;

	ID3D10Texture3D* texture;
	HRESULT hr = device->CreateTexture3D(&desc, NULL, &texture);	
	
	if (FAILED(hr)) return NULL;
	else			return texture;
}


void TextureUtils::SaveScreenShot( ID3D10Texture2D* source, ID3D10Texture2D* destination, std::string filename, ID3D10Device* device )
{
	device->CopyResource(destination, source);
	D3DX10SaveTextureToFileA(destination, D3DX10_IFF_PNG, filename.c_str());	
}
