#pragma once

#include "VertexDefinition.h"

class VertexBufferUtils {
public:
	template <class T> static ID3D10InputLayout* CreateInputLayout(ID3D10Device* device, ID3D10EffectPass* pass) 
	{
		ID3D10InputLayout* inputLayout = NULL;

		D3D10_PASS_DESC desc;
		pass->GetDesc(&desc);

		device->CreateInputLayout(
				T::InputDesc, 
				T::InputElementCount, 
				desc.pIAInputSignature, 
				desc.IAInputSignatureSize, 
				&inputLayout
		);

		return inputLayout;
	}

	template <class T> static ID3D10Buffer* CreateDefaultVertexBuffer(ID3D10Device* device, T* vertexData, int vertexCount, bool bindToSO = false) 
	{	

		// Specify how what kind of buffer we'd like
		D3D10_BUFFER_DESC vb_desc;
		vb_desc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
		vb_desc.ByteWidth = sizeof(T) * vertexCount;
		vb_desc.CPUAccessFlags = 0;
		vb_desc.MiscFlags = 0;
		vb_desc.Usage = D3D10_USAGE_DEFAULT;

		if (bindToSO) vb_desc.BindFlags |= D3D10_BIND_STREAM_OUTPUT;

		// Specify where to find initial data. (the other struct members are ignored for buffers)
		D3D10_SUBRESOURCE_DATA subResource;
		subResource.pSysMem = vertexData;

		ID3D10Buffer* buffer = NULL;
	
		if (FAILED(device->CreateBuffer(&vb_desc, &subResource, &buffer))){
			return NULL;
		}
		return buffer;
	}

	template <class T> static ID3D10Buffer* CreateSOVertexBuffer(ID3D10Device* device, int vertexCount)
	{
		ID3D10Buffer *buffer;

		D3D10_BUFFER_DESC bufferDesc =
		{
			sizeof(T) * vertexCount,
			D3D10_USAGE_DEFAULT,
			D3D10_BIND_STREAM_OUTPUT | D3D10_BIND_VERTEX_BUFFER,
			0,
			0
		};
		if (FAILED(device->CreateBuffer(&bufferDesc, NULL, &buffer))) {
			return NULL;
		}
		return buffer;
	}

	template <class T> static void SetSingleVertexBuffer(ID3D10Device* device, ID3D10Buffer* vertexBuffer)
	{
		UINT stride = sizeof(T);
		UINT offset = 0;
		device->IASetVertexBuffers(0,1, &vertexBuffer, &stride, &offset);
	}

	static void SetSingleSOVertexBuffer(ID3D10Device* device, ID3D10Buffer* vertexBuffer)
	{
		UINT offset[1] = { 0 };
		device->SOSetTargets(1, &vertexBuffer, offset);
	}
};
