#pragma once

#include "DXUT.h"
#include "DrawableGameComponent.h"
#include "MessageHandler.h"
#include "IKeyHandler.h"

#pragma warning (disable:4995)
#include <list>

class MainFrame
{
public:
	static MainFrame* Instance() { 
		static MainFrame mainFrame;
		return &mainFrame;
	}

private:
	MainFrame();
	~MainFrame();
public:
	void ShutDown(int errorCode = 0);

	HRESULT OnDeviceCreated(ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC *pBufferSurfaceDesc);
	void OnDeviceDestroyed();

	HRESULT OnSwapChainResized(ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBufferSurfaceDesc);
	void OnSwapChainReleasing();

	void OnFrameUpdate(double time, float elapsedTime);
	void OnFrameRender(ID3D10Device* pd3dDevice, double fTime, float fElapsedTime);

	void OnKeyDown(UINT nChar, bool keyDown, bool altDown);
	void OnMouseEvent();

	void MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void AddComponent(GameComponent* component);
	void AddMessageHandler(MessageHandler* handler);
	void AddKeyHandler(IKeyHandler* handler);
	
	void RemoveComponent(GameComponent* component);
private:	
	void SortComponents();
	void AddObjectReference(ReferencedObject* objectToBeResponsibleOf);
	void ClearDeadObjects();
	std::list<ReferencedObject*> referencedObjects;
	std::list<ReferencedObject*> deadObjects;

	std::list<DrawableGameComponent*> drawableGameComponents;
	std::list<GameComponent*> gameComponents;
	std::list<MessageHandler*> messageHandlers;
	std::list<IKeyHandler*> keyHandlers;

	bool shouldShutDown;
	int shutDownErrorCode;
};


class MainFrameInitializer
{
public:
	bool InitializeMainFrame(MainFrame& mainFrame);
};
