using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework.Graphics;

namespace KolmogorovTurbulenceGenerator
{
    
    public partial class GUI : Form
    {
        private class TextureSize
        {
            public int Size;
            public int Dimension;
            public TextureSize(int size, int dimension)
            {
                this.Size = size;
                this.Dimension = dimension;
            }
            public override string ToString()
            {
                string ret = "";
                for (int i = 0; i < Dimension-1; i++)
                {
                    ret += Size + "x";
                }
                ret += Size;
                return ret;
            }

        }

        public GUI()
        {
            InitializeComponent();

            this.textureSizeCombo.DataSource = new TextureSize[] { 
                new TextureSize(32,2),
                new TextureSize(64,2),
                new TextureSize(128,2),
                new TextureSize(32,3),
                new TextureSize(64,3),
                new TextureSize(128,3)
            };

            this.Load += new EventHandler(GUI_Load);
        }

        void GUI_Load(object sender, EventArgs e)
        {
            this.InitializeGraphics();
        }

        private void saveTextureButton_Click(object sender, EventArgs e)
        {
            this.saveFileDialog.ShowDialog();
            if (this.saveFileDialog.FileName.Length == 0) return;

            TextureSize textureSize = (TextureSize) this.textureSizeCombo.SelectedItem;
            Generator generator = null;
            if (textureSize.Dimension == 2)
            {
                generator = new Generator2D(textureSize.Size, textureSize.Size,
                                            (double)this.epsilonNumericUpDown.Value, 
                                            (double)this.intertalFrequencyNumericUpDown.Value, 
                                            (double)this.sigmaNumericUpDown.Value);
            }
            else
            {
                generator = new Generator3D(textureSize.Size, textureSize.Size, textureSize.Size,
                                            (double)this.epsilonNumericUpDown.Value, 
                                            (double)this.intertalFrequencyNumericUpDown.Value, 
                                            (double)this.sigmaNumericUpDown.Value);
            }
            generator.GenerateTurbulenceTexture(this.saveFileDialog.FileName, this.viewFriendlyCheckBox.Checked);
            MessageBox.Show("Done");
        }

        private void InitializeGraphics()
        {
            Form form = new Form();
            GraphicsDevice device = new GraphicsDevice(
                GraphicsAdapter.DefaultAdapter,
                DeviceType.Hardware,
                form.Handle,
                CreateOptions.HardwareVertexProcessing,
                new PresentationParameters());
            TextureSaver.GraphicsDevice = device;
        }

        private void textureSizeCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            TextureSize size = (TextureSize) this.textureSizeCombo.SelectedItem;
            if (size.Dimension == 2)
            {
                this.saveFileDialog.Filter = "DirectDraw Surface(*.dds)|*.dds|JPEG (*.jpg)|*.jpg";
            }
            else
            {
                this.saveFileDialog.Filter = "DirectDraw Surface(*.dds)|*.dds";
            }
        }

        private void exitButton_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

    }
}