using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System.Windows.Forms;

namespace KolmogorovTurbulenceGenerator
{
    class Texture2DSaver : TextureSaver
    {
        private Texture2D texture;
        Vector4[] data = null;
        public Texture2DSaver(int width, int height)
        {
            texture = new Texture2D(Texture2DSaver.GraphicsDevice, width, height, 1, ResourceUsage.None, SurfaceFormat.Vector4);
            data = new Vector4[texture.Width * texture.Height];
        }

        public int Width
        {
            get { return texture != null ? texture.Width : -1; }
        }

        public int Height
        {
            get { return texture != null ? texture.Height : -1; }
        }

        public void SetValues(int x, int y, float r, float g, float b)
        {
            SetValue(x, y, Component.R, r);
            SetValue(x, y, Component.G, g);
            SetValue(x, y, Component.B, b);
        }

        public void SetValue(int x, int y, Component c, float value)
        {
            if (x < 0 || x >= texture.Width || y < 0 || y >= texture.Height)
            {
                throw new ArgumentOutOfRangeException();
            }
            switch (c)
            {
                case Component.R:
                    data[y * texture.Width + x].X = value;
                    break;
                case Component.G:
                    data[y * texture.Width + x].Y = value;
                    break;
                case Component.B:
                    data[y * texture.Width + x].Z = value;
                    break;
            }
        }

        public void SaveTexture(string filename)
        {
            texture.SetData(data);

            ImageFileFormat format = ImageFileFormat.Jpg;
            if (filename.EndsWith(".dds")) format = ImageFileFormat.Dds;

            texture.Save(filename, format);
        }
    }

}
