using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace KolmogorovTurbulenceGenerator
{
    class Texture3DSaver : TextureSaver
    {
        private Texture3D texture;
        private Vector4[] data = null;
        
        public Texture3DSaver(int width, int height, int depth)
        {
            texture = new Texture3D(Texture2DSaver.GraphicsDevice, width, height, depth, 1, ResourceUsage.None, SurfaceFormat.Vector4);
            data = new Vector4[texture.Width * texture.Height * texture.Depth];
        }

        public int Width
        {
            get { return texture != null ? texture.Width : -1; }
        }

        public int Height
        {
            get { return texture != null ? texture.Height : -1; }
        }

        public int Depth
        {
            get { return texture != null ? texture.Depth : -1; }
        }

        public void SetValues(int x, int y, int z, float r, float g, float b)
        {
            SetValue(x, y, z, Component.R, r);
            SetValue(x, y, z, Component.G, g);
            SetValue(x, y, z, Component.B, b);
        }

        public void SetValue(int x, int y, int z, Component c, float value)
        {
            if (x < 0 || x >= texture.Width || y < 0 || y >= texture.Height || z < 0 || z >= texture.Depth)
            {
                throw new ArgumentOutOfRangeException();
            }
            switch (c)
            {
                case Component.R:
                    data[z * texture.Width*texture.Height + y * texture.Width + x].X = value;
                    break;
                case Component.G:
                    data[z * texture.Width * texture.Height + y * texture.Width + x].Y = value;
                    break;
                case Component.B:
                    data[z * texture.Width * texture.Height + y * texture.Width + x].Z = value;
                    break;
            }
        }

        public void SaveTexture(string filename)
        {
            texture.SetData(data);

            ImageFileFormat format = ImageFileFormat.Dds;
            if (!filename.EndsWith(".dds"))
            {
                throw new ArgumentException("Only .dds files are supported when generating 3D textures");
            }

            texture.Save(filename, format);
        }
    }
}
