// This is the main DLL file.

#include "stdafx.h"

#include "SimpleFFTWWrapper.h"

namespace SimpleFFTWWrapper {

	FFTWComplex2D::FFTWComplex2D(int nx, int ny, FFTTransformType type)
	{
		this->nx = nx;
		this->ny = ny;
		in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * nx * ny);
		out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * nx * ny);
	}
	
	void FFTWComplex2D::transform() 
	{
		fftw_plan p = fftw_plan_dft_2d(nx, ny, in, out, FFTW_BACKWARD, FFTW_ESTIMATE);
		fftw_execute(p);
		fftw_destroy_plan(p);
	}	

	FFTWComplex2D::~FFTWComplex2D() 
	{
		this->!FFTWComplex2D();
		GC::SuppressFinalize(this);
	}

	FFTWComplex2D::!FFTWComplex2D() 
	{
		Console::WriteLine("FFTWComplex2D::!FFTWComplex2D");
		fftw_free(in); 
		fftw_free(out);
	}

	void FFTWComplex2D::zero() 
	{
        for (int x = 0; x < nx; x++)
        {
            for (int y = 0; y < ny; y++)
            {
                setInput(x, y, 0, 0);
            }
        }
	}
	double FFTWComplex2D::getRealInput(int x, int y)
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny) {
			return in[x*ny+y][0];
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	double FFTWComplex2D::getImagInput(int x, int y)
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny) {
			return in[x*ny+y][1];
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	double FFTWComplex2D::getRealOutput(int x, int y)
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny) {
			return out[x*ny+y][0];
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	double FFTWComplex2D::getImagOutput(int x, int y)
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny) {
			return out[x*ny+y][1];
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	void FFTWComplex2D::setInput(int x, int y, double real, double imag) 
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny) {
			in[x*ny+y][0] = real;
			in[x*ny+y][1] = imag;
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	///////////////////////// 3D ///////////////////////////////////

	FFTWComplex3D::FFTWComplex3D(int nx, int ny, int nz, FFTTransformType type)
	{
		this->nx = nx;
		this->ny = ny;
		this->nz = nz;
		in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * nx * ny * nz);
		out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * nx * ny * nz);
	}
	
	void FFTWComplex3D::transform() 
	{
		fftw_plan p = fftw_plan_dft_3d(nx, ny, nz, in, out, FFTW_BACKWARD, FFTW_ESTIMATE);
		fftw_execute(p);
		fftw_destroy_plan(p);
	}	

	FFTWComplex3D::~FFTWComplex3D() 
	{
		this->!FFTWComplex3D();
		GC::SuppressFinalize(this);
	}

	FFTWComplex3D::!FFTWComplex3D() 
	{
		//Console::WriteLine("FFTWComplex3D::!FFTWComplex3D");
		fftw_free(in); 
		fftw_free(out);
	}

	void FFTWComplex3D::zero() 
	{
        for (int x = 0; x < nx; x++)
        {
            for (int y = 0; y < ny; y++)
            {
				for (int z = 0; z < nz; z++)
				{
					setInput(x, y, z, 0, 0);
				}
            }
        }
	}
	double FFTWComplex3D::getRealInput(int x, int y, int z)
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny && z >= 0 && z < nz) {
			return in[z*nx*ny+x*ny+y][0];
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	double FFTWComplex3D::getImagInput(int x, int y, int z)
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny && z >= 0 && z < nz) {
			return in[z*nx*ny+x*ny+y][1];
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	double FFTWComplex3D::getRealOutput(int x, int y, int z)
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny && z >= 0 && z < nz) {
			return out[z*nx*ny+x*ny+y][0];
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	double FFTWComplex3D::getImagOutput(int x, int y, int z)
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny && z >= 0 && z < nz) {
			return out[z*nx*ny+x*ny+y][1];
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

	void FFTWComplex3D::setInput(int x, int y, int z, double real, double imag) 
	{
		if (x >= 0 && x < nx && y >= 0 && y < ny && z >= 0 && z < nz) {
			in[z*nx*ny+x*ny+y][0] = real;
			in[z*nx*ny+x*ny+y][1] = imag;
		} else {
			throw gcnew IndexOutOfRangeException();
		}
	}

};